/*****************************************************************************
  klocate -- a KDE front-end to slocate
  version 0.50.3, March 7, 2004                                                              
  Programmed by: Alexandre Oberlin                              
  Report any Bugs to: alxobr@free.fr   
  Thanks to Kevin Lindsay for programming slocate.                                                                                                                                      
     Copyright (C) version Free Software Foundation, Inc.                       
     This program is free software; you can redistribute it and/or modify    
     it under the terms of the GNU General Public License as published by    
     the Free Software Foundation; either version 2, or (at your option)     
     any later version.                                                      
     This program is distributed in the hope that it will be useful,         
     but WITHOUT ANY WARRANTY; without even the implied warranty of          
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
     GNU General Public License for more details.                            
     You should have received a copy of the GNU General Public License       
     along with this program; if not, write to the Free Software             
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               
*****************************************************************************/
/****************************************************************
** disklist.cpp
** parse utils, classes DiskList, VolumeList, 
** VolumeEntry, DiskEntry, GuessGetString, GuessGetStringandPath, GuessGetPath, 
** GetUpdateList, 
** DrivesTab, GeneralTab, OptionsDB, DisplayCommand, SystemThread
** TableMessage
****************************************************************/
#include <errno.h>
#include <grp.h>  
#include <pwd.h>  
#include <time.h>  
#include <sys/stat.h>  
#include <libgen.h>  
#include "disklist.h"
#include "topwidget.h"
#include <qregexp.h> 
#include <qtooltip.h>
#include <qfileinfo.h>
#include <klocale.h>
extern char * stderrtmp;
extern char * stdouttmp;
extern TopWidget * tw;
int skip_whitespace(const char * inputline, int * pti)
{
  if (!inputline[*pti]) return *pti;
  while (inputline[*pti] == '\r' ||  inputline[*pti] == '\n' ||  inputline[*pti] == '\t' \
	 ||  inputline[*pti] == ' ') (*pti)++;
  return *pti;
}
int skip_non_whitespace(const char * inputline, int * pti)
{
  while (inputline[*pti]  && inputline[*pti] != '\r' &&  inputline[*pti] != '\n' &&  inputline[*pti] != '\t' \
	 &&  inputline[*pti] != ' ') (*pti)++;
  return *pti;
}
int get_token (QString & output, int n, const char * inputline, const char * defaut)
{
  int i, nb_read, current_begin, current_end;
  i=0; nb_read=0;
  if (inputline == 0)  
    { 
      output = QString(defaut);
      return 0;
    }
  while (inputline[i] && nb_read < n)
    {
      i=skip_whitespace(inputline, &i);
      current_begin=i;
      i=skip_non_whitespace(inputline, &i);
      current_end=i;
      if (current_end == current_begin) break; 
      nb_read++;
      if (nb_read == n) 
	{  
	  output = QString(inputline).mid(current_begin, current_end - current_begin);
	  if ( (output.upper() == "DEFAULT")  || (output.upper() == "DEFAULTS") || (output == "0"))
	    break;
	  else
	    return i;
	}
    }
  output = QString(defaut);
  return i;
}
VolumeEntry::VolumeEntry(const QString _filename, const QString _directoryname)
  : filename (_filename), directoryname (_directoryname)
{
  if (_filename.right(3).lower() == ".rw") 
    writeflag = true;
  else if  (_filename.right(3).lower() == ".ro")
    writeflag = false;
}
QString VolumeEntry::displayname()
{
 return filename.left(filename.length()-3);
}
DiskEntry::DiskEntry(const char * _mntpnt, const char * resvalue)
{
  get_token(mntpnt, 1, _mntpnt, "Unknown");
  if (mntpnt.right(1) != "/") mntpnt += "/";
  QString temp;
  get_token(temp, 1, resvalue, "FIX");
  temp=temp.upper();
  if (temp == "REM")
    remflag = true;
  else if (temp == "FIX")
    remflag = false;
  else
    {
      QString mess;
      mess = i18n("Error in %1:\n rem tag for %2 should be FIX or REM. \nApplication will exit.").arg(resfile).arg(mntpnt);
      KMessageBox::information( 0, mess,  i18n("Fatal error")  );
      MUSTQUIT=true;
    }
  get_token(excldirs, 2, resvalue, "");
  get_token(displayname, 3, resvalue, "Unknown");
  get_token(temp, 4, resvalue, "rw");
  temp=temp.lower();
  if (temp == "rw")
    writeflag = true;
  else if (temp == "ro")
    writeflag = false;
  else
    {
      QString mess;
      mess  = i18n("Error in %1:\n write tag for %2 should be rw or ro. \nApplication will exit.").arg(resfile).arg(mntpnt);
      KMessageBox::information( 0, mess,  i18n("Fatal error"));
      MUSTQUIT=true;
    }
}
DiskList::DiskList( QWidget *parent, const char *name)
  : QListBox( parent, name )
{
  setHScrollBarMode( QScrollView::AlwaysOff);
  int nb_entries=read_entries() ;
  setSelectionMode( Extended );
  for (int i = 0; i< nb_entries; i++)
    {
      new QListBoxText (this, QString(entries[i]->displayname));
    }
QToolTip::add( this, i18n("List of currently mounted drives.") + "\n" + i18n("Select those whose contents should be updated.") );
}
DiskList::~DiskList()
{
  for (uint i = 0; i < count(); i++)
    {
      delete entries[i];
    }   
}
void DiskList::showEvent ( QShowEvent * ) 
{
  setMinimumHeight (contentsHeight() + 2 * frameWidth());
}
int DiskList::read_entries()
{
  int nb_entries = 0;
  QString value;
  FILE * mtab_fd = setmntent (MOUNTED, "r"); 
  struct mntent * mtab_struct;
  kapp->config()->setGroup ("Mounts");
  while ((mtab_struct = getmntent(mtab_fd)) != NULL)
    {
      value = kapp->config()->readEntry(mtab_struct->mnt_dir);
      if (! value.isEmpty())
	{
	    entries[nb_entries++]= new DiskEntry(mtab_struct->mnt_dir, value);
	}
    }
  endmntent(mtab_fd); 
  return nb_entries;
}
VolumeList::VolumeList( QWidget *parent, const char *name)
  : QListBox( parent, name )
{
  int nb_entries=read_entries();
  setSelectionMode( Extended );
  for (int i = 0; i< nb_entries; i++)
    new QListBoxText (this, QString(entries[i]->displayname()));
  sort(true);
  id = new int [ 3 ];
  popup = new QPopupMenu( this );
  CHECK_PTR( popup );
  id[0]=popup->insertItem( i18n("Delete DB"), this, SLOT(deleteDB()) );
  id[1]=popup->insertItem( i18n("Rename DB"), this, SLOT(renameDB()) );
  id[2]=popup->insertItem( i18n("DB file info"), this, SLOT(statDB()) );
  QToolTip::add( this, i18n("List of index files.") + "\n" + i18n("Right-click on an entry to get a menu for that entry.") );
  connect( this, SIGNAL(rightButtonPressed ( QListBoxItem *, const QPoint & )), SLOT(displayPopup(QListBoxItem *, const QPoint &)) );
}
void VolumeList::displayPopup(QListBoxItem * lbi, const QPoint & point)
{
  if (lbi == 0)
    return;  
  else
    {
      contextMenuTarget= lbi;  
      popup->exec(point);
    }
}
VolumeList::~VolumeList()
{
  for (uint i = 0; i < count(); i++)
    {
      delete entries[i];
    }
  delete popup;
  delete [] id;
}
void  VolumeList::sort ( bool ascending)
{
  int gap, i, j, n=count();
  VolumeEntry *temp;
  for (gap = n/2; gap > 0; gap /=2)
    for (i = gap; i < n; i++)
      for(j=i-gap; j >= 0 ; j -= gap)
	{
#if defined(KDE3)
	  if (QString::localeAwareCompare(entries[j]->displayname(), entries[j+gap]->displayname()) <= 0) break;
#else  
	  if (QString::compare(entries[j]->displayname(), entries[j+gap]->displayname()) <= 0) break;
#endif
	  temp = entries[j];
	  entries[j] = entries[j+gap];
	  entries[j + gap] = temp;
	}
  QListBox::sort(ascending);
}
int VolumeList::read_entries()
{
  QString result;
  kapp->config()->setGroup ("Options");
  result = kapp->config()->readEntry ("MAX_LABELS");
  if (!result)
    MAX_LABELS=DEF_MAX_LABELS;
  else
    {
      bool ok;
      MAX_LABELS=result.toUInt(&ok);
      if (!ok)
	{
	  QString msg;
	  msg =i18n("Error in %1: entry %2 should be a number.").arg(resfile).arg("MAX_LABELS");
	  KMessageBox::information( 0, msg,  i18n("Warning"));
	  MAX_LABELS=DEF_MAX_LABELS;
	}	
    }
  entries = new VolumeEntry * [MAX_LABELS]; 
  int nb_entries = 0;
  result = kapp->config()->readEntry ("DBPATH");
  if (!result)  
    result = "/var/lib/xlocate/:/var/lib/slocate/:" + (QString)getenv("HOME") +  "/.kde/share/apps/klocate/";
  DBPATH=QStringList::split(":", result, false);  
  QStringList to_remove;
  int readresult;
  for (QStringList::Iterator it = DBPATH.begin();   it != DBPATH.end(); ++it)
    {
      if ((*it).right(1) != "/")  *it += "/";  
      readresult =  readDBDIR(*it, nb_entries);
      if (readresult == -1)
	to_remove.prepend(*it);
      else
	nb_entries += readresult;
    }
#if defined(KDE3)
  if (to_remove.empty())
#else  
  if (to_remove.isEmpty())
#endif
    {
    }
  else
    {
    }
#if defined(KDE3)
  if (DBPATH.empty())
#else  
  if (DBPATH.isEmpty())
#endif
    {
      QString mesg = i18n("Invalid DBPATH %1.\nCheck your config file %2.").arg(result).arg(resfile);
      KMessageBox::information(0, mesg,  i18n("Fatal error"));
      MUSTQUIT=true;
    }
  for (QStringList::Iterator it = to_remove.begin();   it != to_remove.end(); ++it)
    {
       DBPATH.remove(*it);
    }
  for (QStringList::Iterator it = DBPATH.begin();   it != DBPATH.end(); ++it)
    {
    }
  return nb_entries;
}
int VolumeList::readDBDIR(QString path, int old_nbe)
{
  uint nb_entries = old_nbe;
  DIR * dbdir;
  struct dirent * direntry;
  if ((dbdir = opendir((const char *)path)) != NULL)  
    {
      QString filename;
      while ((direntry = readdir(dbdir)) != NULL)  
	{
	  filename = (QString)(direntry->d_name);
	  if (filename.right(3).lower() != ".rw" && filename.right(3).lower() != ".ro")
	    continue;  
	  if (nb_entries < MAX_LABELS)
	    {
	      entries[nb_entries++]= 
		new VolumeEntry(filename, path);
	    }
	  else
	    {
	      QString msg;
	      msg = i18n("Some index files in %1 could not be loaded.\n Try to increase MAX_LABELS in %2.").arg(path).arg(resfile);
	      KMessageBox::information( 0, msg,  i18n("Warning")); 
	      break;
	    }
	}
      closedir(dbdir);
    }
  else  
    {
      QString mess = QString(strerror(errno));
      KMessageBox::information(0 , i18n("Could not open directory %1: %2").arg(path).arg(mess) + "\n" + i18n("Trying to make it"),  i18n("Warning"));      
      if ( mkdir ((const char *)path, S_IRWXU) != 0)  
	{
	  mess = QString(strerror(errno));
	  KMessageBox::information(0,i18n("Could not create directory %1: %2").arg(path).arg(mess), i18n("Warning"));
	  return -1;
	}
    }
  return nb_entries - old_nbe;
}
void VolumeList::adjoinLabel ( QString & new_label, QString & new_directoryname, bool writeflag) 
{
  if ( count() < MAX_LABELS)
    {
      entries[count()]= new  VolumeEntry(new_label +  (writeflag ? ".rw" : ".ro"), new_directoryname);  
      new QListBoxText (this, new_label);  
      sort();
    }
  else
    {
      QString msg= i18n("Could not add new entry in volume list.\n Try to increase MAX_LABELS in %1.").arg(resfile);
      KMessageBox::information( 0, msg,  i18n("Warning")); 
    }
}
void VolumeList::delSelectedDBs () 
{
  int nb_deleted=0, nb_selected=0;
  for (int i = count()-1 ; i >= 0; i--)  
    {
      if (isSelected(i))
	{
	  nb_selected++;
	  if (delDB(i)) nb_deleted++;
	}
    }
  if (nb_selected == 0) 
    {
      KMessageBox::information(this , i18n("Please select volumes to delete in the volume list."),  i18n("Tip"));
      return;
    }
  if (nb_deleted != nb_selected)
    {
      QString mess = i18n("Only %1 DB files deleted instead of %2").arg(nb_deleted).arg(nb_selected);
      KMessageBox::information(parentWidget(), mess,  i18n("Report"));    
    }
}
bool VolumeList::delDB (int i) 
{
  QString current_filename=entries[i]->filename;
  QString current_db=entries[i]->directoryname + current_filename;
  if (KMessageBox::warningContinueCancel
      (parentWidget(), 
       i18n("Are you sure you want to delete entry `%1\'? \nExcept for slinks, the associated DB file will be deleted as well.").arg(entries[i]->displayname()),
        i18n("Confirmation"), i18n("Proceed"))
      ==  KMessageBox::Cancel)
    {
      KMessageBox::information(parentWidget(), i18n("Deletion of volume `%1\' canceled.").arg(entries[i]->displayname()),  i18n("Report"));
      return false;
    }
  else
    {
      if (remove(current_db) != EXIT_SUCCESS)
	{
	  QString mess = QString(strerror(errno));
	  KMessageBox::information(parentWidget() , i18n("Deletion of file %1 failed: %2").arg(current_db).arg(mess),  i18n("Warning"));
	  return false;
	}
      else
	{
	  removeItem(i);  
	  delete entries[i];
	  entries[i]=(VolumeEntry *)NULL;
	  for (uint j=i ; j < count() ;  j++) 
	    entries[j]=entries[j+1];
	  return true;
	}
    }	
  tw->selbutts->rementry(current_filename);
}
bool VolumeList::deleteDB ()  
{
  return delDB(index(contextMenuTarget));
}
bool VolumeList::renDB (QString oldname, QString newname, bool rw)
{  
  bool result = false;
  for (QStringList::Iterator it = DBPATH.begin();   it != DBPATH.end(); ++it)
    {
      if (renDB(oldname,  newname, rw, *it)) result = true;
    }
  return result;
}
bool VolumeList::renDB (QString oldname, QString newname, bool rw, QString directoryname) 
{
  QString extension = (rw ? ".rw" : ".ro");
  QString old_db = directoryname + oldname + extension; 
  QString new_db = directoryname + newname + extension;
  bool ret=true; 
  for (int i = count()-1 ; i >= 0; i--)  
    {
      if (entries[i]->displayname() == oldname && entries[i]->writeflag == rw)
	{
	  if (rename(old_db, new_db) != EXIT_SUCCESS)
	    {
	      QString mess = QString(strerror(errno));
	      KMessageBox::information(parentWidget() , i18n("Renaming of file %1 failed: %2").arg(old_db).arg(mess),  i18n("Warning"));
	      ret=false;
	    }
	  else
	    {
	      entries[i]->filename=newname + extension;
	      changeItem(newname, i);
	      ret=true;
	    }
	  break;
	}
    }
  sort();
  tw->selbutts->rementry(oldname);
  return ret;
}
bool VolumeList::renameDB ()  
{
  int dbi = index(contextMenuTarget);
  bool ret;
  QString oldname = entries[dbi]->displayname();
  QString directoryname = entries[dbi]->directoryname;
  GuessGetString * gndb = new GuessGetString(this, oldname, i18n("Enter new name for DB entry:"));
  int result = gndb->exec();
  if (result == QDialog::Accepted) 
    {
      ret = renDB (oldname, gndb->le->text(), entries[dbi]->writeflag, directoryname);
    }
  else
    {
      KMessageBox::information(parentWidget(), i18n("Renaming of volume `%1\' canceled.").arg(oldname),  i18n("Report"));
      ret = false;
    }  
  delete gndb;
  return ret;
}
void VolumeList::setSel()
{
  QStringList cur_preset = tw->cur_preset;
#if defined(KDE3)
  if (cur_preset.empty()) return;
#else  
  if (cur_preset.isEmpty()) return;
#endif
  clearSelection();
  addToSel();
}
void VolumeList::addToSel()
{
  QStringList cur_preset = QStringList(tw->cur_preset);
#if defined(KDE3)
  if (cur_preset.empty()) 
#else  
  if (cur_preset.isEmpty())
#endif
    {
      return;
    }
  uint i;
#if defined(KDE3)
  for (i = 0 ; i  < count() && !cur_preset.empty() ; i++)
#else  
  for (i = 0 ; i  < count() && !cur_preset.isEmpty() ; i++)
#endif
    {
      for (QStringList::Iterator it = cur_preset.begin();   it != cur_preset.end(); ++it)
	if (entries[i]->filename == *it) 
	  {
	    setSelected (i, true);
	    cur_preset.remove(it);
	    break;
	  }
	else if (entries[i]->filename > *it)
	  {
	  }
	else
	  {
	  }
    }
  setBottomItem(i);
}
/**************************************************************************
****************** MODE_STRING from fileutils filemode.c ******************
**************************************************************************/
/* Set the 's' and 't' flags in file attributes string CHARS,
   according to the file mode BITS.  */
static void
setst (mode_t bits, char *chars)
{
#ifdef S_ISUID
  if (bits & S_ISUID)
    {
      if (chars[3] != 'x')
	/* Set-uid, but not executable by owner.  */
	chars[3] = 'S';
      else
	chars[3] = 's';
    }
#endif
#ifdef S_ISGID
  if (bits & S_ISGID)
    {
      if (chars[6] != 'x')
	/* Set-gid, but not executable by group.  */
	chars[6] = 'S';
      else
	chars[6] = 's';
    }
#endif
#ifdef S_ISVTX
  if (bits & S_ISVTX)
    {
      if (chars[9] != 'x')
	/* Sticky, but not executable by others.  */
	chars[9] = 'T';
      else
	chars[9] = 't';
    }
#endif
}
/* Return a character indicating the type of file described by
   file mode BITS:
   'd' for directories
   'D' for doors
   'b' for block special files
   'c' for character special files
   'n' for network special files
   'm' for multiplexor files
   'M' for an off-line (regular) file
   'l' for symbolic links
   's' for sockets
   'p' for fifos
   'C' for contigous data files
   '-' for regular files
   '?' for any other file type.  */
static char
ftypelet (mode_t bits)
{
#ifdef S_ISBLK
  if (S_ISBLK (bits))
    return 'b';
#endif
  if (S_ISCHR (bits))
    return 'c';
  if (S_ISDIR (bits))
    return 'd';
  if (S_ISREG (bits))
    return '-';
#ifdef S_ISFIFO
  if (S_ISFIFO (bits))
    return 'p';
#endif
#ifdef S_ISLNK
  if (S_ISLNK (bits))
    return 'l';
#endif
#ifdef S_ISSOCK
  if (S_ISSOCK (bits))
    return 's';
#endif
#ifdef S_ISMPC
  if (S_ISMPC (bits))
    return 'm';
#endif
#ifdef S_ISNWK
  if (S_ISNWK (bits))
    return 'n';
#endif
#ifdef S_ISDOOR
  if (S_ISDOOR (bits))
    return 'D';
#endif
#ifdef S_ISCTG
  if (S_ISCTG (bits))
    return 'C';
#endif
  /* The following two tests are for Cray DMF (Data Migration
     Facility), which is a HSM file system.  A migrated file has a
     `st_dm_mode' that is different from the normal `st_mode', so any
     tests for migrated files should use the former.  */
#ifdef S_ISOFD
  if (S_ISOFD (bits))
    /* off line, with data  */
    return 'M';
#endif
#ifdef S_ISOFL
  /* off line, with no data  */
  if (S_ISOFL (bits))
    return 'M';
#endif
  return '?';
}
/* Like filemodestring, but only the relevant part of the `struct stat'
   is given as an argument.  */
void
mode_string (mode_t mode, char *str)
{
  str[0] = ftypelet (mode);
  str[1] = mode & S_IRUSR ? 'r' : '-';
  str[2] = mode & S_IWUSR ? 'w' : '-';
  str[3] = mode & S_IXUSR ? 'x' : '-';
  str[4] = mode & S_IRGRP ? 'r' : '-';
  str[5] = mode & S_IWGRP ? 'w' : '-';
  str[6] = mode & S_IXGRP ? 'x' : '-';
  str[7] = mode & S_IROTH ? 'r' : '-';
  str[8] = mode & S_IWOTH ? 'w' : '-';
  str[9] = mode & S_IXOTH ? 'x' : '-';
  setst (mode, str);
}
/**************************************************************************
************** END of MODE_STRING from fileutils filemode.c ***************
**************************************************************************/
char * readlink_malloc (const char *filename)  
{
  int size =  100;          
  while (1)
    {
      char *buffer = (char *) malloc (size);
      int nchars = readlink (filename, buffer, size);
      if (nchars == -1)
	  return NULL;
      if (nchars < size)
	{
	  buffer[nchars]='\0';
	  return buffer;
	}
      free (buffer);
      size *= 2;
    }
}
void VolumeList::statDB ()  
{
  int dbi = index(contextMenuTarget);
  QString dbfile = entries[dbi]->directoryname + entries[dbi]->filename;
  QString argue = "%1";
  struct stat *buf;
  buf = (struct stat *) malloc (sizeof (struct stat));
  if (lstat(dbfile, buf) == -1)
    {
      QString mess = QString(strerror(errno));
      KMessageBox::information( 0, i18n("Could not lstat %1: %2").arg(dbfile).arg(mess),  i18n("Warning"));
    }
  else
    {
      QString header = i18n("Stats for file %1").arg(dbfile);
      char perms[11];
      mode_string (buf->st_mode, perms);
      perms[10] = '\0';		/* `mode_string' does not null terminate. */
      QStringList messlist ;
      messlist.append(i18n("type and permissions:"));
      messlist += QString(perms);
      if (S_ISLNK (buf->st_mode))
	{
	  char * target = readlink_malloc(dbfile.latin1());
	  if (target == NULL)
	    {
	      QString mess = QString(strerror(errno));
	      KMessageBox::information( 0, i18n("Could not read symlink %1: %2").arg(dbfile).arg(mess),  i18n("Warning"));
	    }
	  else
	    {
	    messlist +=i18n("symlink pointing to:");
	    messlist +=QString(target);
	    }
	  free (target);
	}
      else 
	{
	  messlist +=i18n("number of hard links:");
	  messlist +=argue.arg(buf->st_nlink);
	}
      struct passwd * theuser = getpwuid (buf->st_uid);
      messlist +=i18n("user:");
      messlist +=argue.arg(theuser->pw_name);
      struct group * thegroup = getgrgid (buf->st_gid);
      messlist +=i18n("group:");
      messlist +=argue.arg(thegroup->gr_name);
      messlist +=i18n("total size, in bytes:");
      messlist +=argue.arg((unsigned long int)buf->st_size);
      QString filetime;
      messlist +=i18n("last access:");
      filetime=QString(ctime(&(buf->st_atime)));
      filetime.truncate(strlen(filetime)-1);
      messlist += filetime;
      messlist +=i18n("last modification:");
      filetime=QString(ctime(&(buf->st_mtime)));
      filetime.truncate(strlen(filetime)-1);
      messlist += filetime;
      messlist +=i18n("last attribute change:");
      filetime=QString(ctime(&(buf->st_ctime)));
      filetime.truncate(strlen(filetime)-1);
      messlist += filetime;
      TableMessage * tb =  new TableMessage( 0,  i18n("DB file statistics"), header, messlist);
      tb->exec();
      delete tb;
    }
  free(buf);
}
void VolumeList::selAll()
{
  selectAll(TRUE);
}
void  VolumeList::customEvent( QCustomEvent * e )
{
  if ( e->type() >= QEvent::User + 1 ) 
    {
      SystemThreadFinishedEvent * ce = (SystemThreadFinishedEvent*)e;
      QString directoryname = QFileInfo(ce->current_db).dirPath(true);
      if (directoryname.right(1) != "/")  directoryname += "/";  
      if (ce->result == 0 && !ce->db_exists) 
	if ( ce->remflag)
	  adjoinLabel(ce->remlabel, directoryname, ce->writeflag);	
	else 
	  adjoinLabel(ce->displayname, directoryname, ce->writeflag);	
    }
}
GuessGetString::GuessGetString( QWidget *parent, QString guesslabel, QString question)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  QLabel *label = new QLabel (question, this);
  whole->addWidget( label, 0, 0);
  whole->addSpacing(GROUPBOX_SPACING );
  le  = new QLineEdit(this);
  le->setMaxLength(MAX_LABEL_LENGTH);
  le->setText(guesslabel);
  setFocusProxy(le);
  whole->addWidget( le, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  QPushButton *ok, *cancel;
  ok = new QPushButton( i18n("OK"), this );
  ok->setDefault(true);
  connect( ok, SIGNAL(clicked()), SLOT(accept()));
  buttons->addWidget(ok, 0, 0);
  cancel = new QPushButton( i18n("Cancel"), this );
  connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
  buttons->addWidget(cancel, 0, 0);
  whole->addLayout(buttons);
}
GuessGetString::~GuessGetString( )
{
}
GuessGetStringandPath::GuessGetStringandPath( QWidget *parent, QString guess1, QString question1, QString guess2, QString question2, QStringList combolist)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  QLabel *label = new QLabel (question1, this);
  whole->addWidget( label, 0, 0);
  whole->addSpacing(GROUPBOX_SPACING );
  le  = new QLineEdit(this);
  le->setMaxLength(MAX_LABEL_LENGTH);
  le->setText(guess1);
  setFocusProxy(le);
  whole->addWidget( le, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QLabel *label2 = new QLabel (question2, this);
  whole->addWidget( label2, 0, 0);
  whole->addSpacing(GROUPBOX_SPACING );
  comb_dbpath = new KComboBox (this);
  comb_dbpath->insertStringList (combolist);
  comb_dbpath->setEditText(guess2);
  whole->addWidget(comb_dbpath, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  QPushButton *ok, *cancel;
  ok = new QPushButton( i18n("OK"), this );
  ok->setDefault(true);
  connect( ok, SIGNAL(clicked()), SLOT(accept()));
  buttons->addWidget(ok, 0, 0);
  cancel = new QPushButton( i18n("Cancel"), this );
  connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
  buttons->addWidget(cancel, 0, 0);
  whole->addLayout(buttons);
}
GuessGetStringandPath::~GuessGetStringandPath( )
{
}
GuessGetPath::GuessGetPath( QWidget *parent, QString guess, QString question, QStringList combolist)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  QLabel *label = new QLabel (question, this);
  whole->addWidget( label, 0, 0);
  whole->addSpacing(GROUPBOX_SPACING);
  comb_dbpath = new KComboBox (this);
  comb_dbpath->insertStringList (combolist);
  comb_dbpath->setEditText(guess);
  whole->addWidget(comb_dbpath, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  QPushButton *ok, *cancel;
  ok = new QPushButton( i18n("OK"), this );
  ok->setDefault(true);
  connect( ok, SIGNAL(clicked()), SLOT(accept()));
  buttons->addWidget(ok, 0, 0);
  cancel = new QPushButton( i18n("Cancel"), this );
  connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
  buttons->addWidget(cancel, 0, 0);
  whole->addLayout(buttons);
}
GuessGetPath::~GuessGetPath( )
{
}
int shellcmd(QString & output, const QString cmd_buf)
{
  FILE *input; 
  char output2[MAXLINE];
  input = popen (cmd_buf, "r");
  if (!input)
    {
      QString mess = i18n("Could not run %1.").arg(cmd_buf);
      KMessageBox::information(0,  mess,  i18n("Error"));
      output="";
      return EXIT_FAILURE;
    }
  fgets(output2, MAXLINE, input);
  output=QString(output2);
  pclose (input);
  return EXIT_SUCCESS;
}
GetUpdateList::GetUpdateList( QWidget *parent)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  disklist = new DiskList( this, "disklist");
  whole->addWidget( disklist, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  update = new QPushButton( i18n("&Update"), this );
  connect( update, SIGNAL(clicked()),  SLOT(update_db()) );
  buttons->addWidget(update, 0, 0);
  done = new QPushButton( i18n("&Done"), this );
  connect( done, SIGNAL(clicked()), SLOT(accept()) );
  buttons->addWidget(done, 0, 0);
  whole->addLayout(buttons);
  setCaption(i18n("Update DBs"));    
}
GetUpdateList::~GetUpdateList( )
{
}
int GetUpdateList::GetRemLabel(QString & remlabel,  QString & dbpath, DiskEntry * current_drive)
{
  bool mounted=false;
  QString mymnt_type, mymnt_fsname, cmd;
  FILE * mtab_fd = setmntent (MOUNTED, "r"); 
  struct mntent * mtab_struct;
  while ((mtab_struct = getmntent(mtab_fd)) != NULL)
    if ((QString(mtab_struct->mnt_dir) + "/") == current_drive->mntpnt)
      {
	mymnt_fsname = QString(mtab_struct->mnt_fsname);
	mymnt_type = QString(mtab_struct->mnt_type);
	mounted=true;
	break;
      }
  endmntent(mtab_fd); 
  if (!mounted) 
    {
      cmd = i18n("Volume %1 not mounted on %2.\nCannot update.").arg(current_drive->displayname).arg(current_drive->mntpnt);
      KMessageBox::information(this, cmd,  i18n("Error"));
      remlabel="";
      return EXIT_FAILURE;
    }	
  else if ((mymnt_type == "vfat") || (mymnt_type == "msdos") || (mymnt_type == "ext2"))
    {
      cmd = "blkid " + mymnt_fsname;
      if (system(cmd) != EXIT_SUCCESS)
	remlabel="";
      else
	{
	  shellcmd(remlabel, cmd); 
	  char * temp = new char[100];
	  char * lab = new char[35];
	  strcpy(temp,  remlabel);
	  strcpy(lab, "rien");
	  lab=strtok(temp, "\" ");
	  while (strcmp (lab=strtok(NULL, "\"= "), "LABEL"))
	    {
	      lab=strtok(NULL, "\" ");
	    }
	  lab=strtok(NULL, "\" ");
	  remlabel=QString(lab);
	}
    }
  else if (mymnt_type == "iso9660")
    {
      FILE * devread;
      if ((devread = fopen((const char * )mymnt_fsname, "r") ) == NULL)
	{
	  QString mess = QString(strerror(errno));
	  KMessageBox::information(parentWidget() , i18n("Could not open %1 for reading: %2").arg(mtab_struct->mnt_fsname).arg(mess),  i18n("Error"));
	  endmntent(mtab_fd); 
	  remlabel="";
	}
      else
	{
	  char thelabel [MAX_CDLABEL_LENGTH + 1];
	  fseek(devread, 0x00008028L, SEEK_SET);
	  for (int k = 0; k<MAX_CDLABEL_LENGTH; k++)
	    thelabel[k]=fgetc(devread);
	  thelabel[MAX_CDLABEL_LENGTH]='\0';
	  fclose(devread);
	  remlabel=((QString)thelabel).stripWhiteSpace();
	}
    }
  else remlabel="";  
  GuessGetStringandPath * gnc;
  remlabel.replace( QRegExp("[/]"), "_" );
  gnc = new GuessGetStringandPath(this, remlabel, i18n("Enter name to index volume:"), dbpath, i18n("Select path for index file:"), DBPATH);
  if ( gnc->exec() ) 
    {
      remlabel = gnc->le->text().stripWhiteSpace();
      dbpath = gnc->comb_dbpath->currentText().stripWhiteSpace();
      delete gnc;
      return EXIT_SUCCESS;
    }
  else 
    {
      cmd = i18n("Update of drive `%1\' canceled.").arg(current_drive->displayname);
      KMessageBox::information(this , cmd, i18n("Report"));
      remlabel="";
      delete gnc;
      return EXIT_FAILURE;
    }
}
int GetUpdateList::GetFixPath(QString & dbpath, DiskEntry * current_drive)
{
  QString cmd;
  GuessGetPath * ggp;
  ggp = new GuessGetPath(this, dbpath, i18n("Enter path for index file:"), DBPATH);
  if ( ggp->exec() ) 
    {
      dbpath = ggp->comb_dbpath->currentText().stripWhiteSpace();
      delete ggp;
      return EXIT_SUCCESS;
    }
  else 
    {
      cmd = i18n("Update of drive `%1\' canceled.").arg(current_drive->displayname);
      KMessageBox::information(this , cmd, i18n("Report"));
      dbpath="";
      delete ggp;
      return EXIT_FAILURE;
    }
}
void GetUpdateList::update_db()
{
  bool nothing_sel = true;
  DiskEntry * current_drive;
  QString remlabel, current_displayname, current_db,  cmd_buffer, dbpath;
  QString final_comment="";
  for (uint i = 0 ; i< disklist->count(); i++)
    {
      if (! disklist->isSelected(i)) continue;
      nothing_sel = false;
      current_drive = disklist->entries[i];
      current_displayname=current_drive->displayname;
      dbpath=DBPATH.first();
      if(current_drive->remflag) 
	{	
	  if (GetRemLabel(remlabel, dbpath, current_drive) == EXIT_SUCCESS)
	    {
	      current_db = dbpath  + remlabel + (current_drive->writeflag ? ".rw" : ".ro");
	    }
	  else 
	    continue;
	}
       else  
  	{
	  if (GetFixPath(dbpath, current_drive) == EXIT_SUCCESS)
	    {
	      current_db = dbpath  + current_displayname + (current_drive->writeflag ? ".rw" : ".ro");
	    }
	  else 
	    continue;
  	} 
      struct stat stats;
      bool db_exists = (stat((const char *)current_db, &stats) == EXIT_SUCCESS);
      if (db_exists && current_drive->remflag && !current_drive->writeflag)
	{
	  cmd_buffer= i18n("A read-only removable volume is already indexed under the name %1.ro.\n Do you want to overwrite it?").arg(remlabel);
	  if (KMessageBox::warningContinueCancel(this,  cmd_buffer, i18n("Confirmation"), i18n("Proceed"))
	      ==  KMessageBox::Cancel)
	    continue;
	}
      if (!db_exists && !current_drive->remflag)
	{
	  cmd_buffer= i18n("No database currently exist under the name %1 in directory %2.\n Do you want to create it?").arg(current_drive->displayname).arg(dbpath);
	  if (KMessageBox::warningContinueCancel(this,  cmd_buffer, i18n("Confirmation"), i18n("Proceed"))
	      ==  KMessageBox::Cancel)
	    continue;
	}
      if (current_drive->excldirs == "")
	cmd_buffer = "slocate " + QString("-l%1 ").arg((tw->seclevel) ? 1 : 0) + "-U " + current_drive->mntpnt
	  + " -o " + "\'" + current_db + "\'";
      else
	cmd_buffer= "slocate " + QString("-l%1 ").arg((tw->seclevel) ? 1 : 0) + "-U " + current_drive->mntpnt\
	  + " -e " + current_drive->excldirs 
	  + " -o " + "\'" + current_db + "\'";
      if (tw->topdebug)
	{
	  DisplayCommand * dpcom;
	  dpcom = new DisplayCommand(this, cmd_buffer, i18n("Edit update command if you wish:"));
	  if ( dpcom->exec() ) 
	    {
	      cmd_buffer = dpcom->le->text().stripWhiteSpace();
	      delete dpcom;
	    }
	  else 
	    {
	      QString cmd = i18n("Update of drive `%1\' canceled.").arg(current_drive->displayname);
	      KMessageBox::information(this , cmd, i18n("Report"));
	      cmd_buffer="";
	      delete dpcom;
	      continue ;
	    }
	}
      (new SystemThread
       (cmd_buffer, tw,
	new SystemThreadFinishedEvent(
				      i, 
				      NULL, 
				      current_drive->displayname,
				      remlabel,
				      current_db,
				      current_drive->writeflag,
				      current_drive->remflag,
				      db_exists
				      )))->start();
    }
  if (nothing_sel) 
    {
      KMessageBox::information(this , i18n("Please select one or more drives to update." ),  i18n("Tip"));
      return;
    }
}
CtrlCheckBox::CtrlCheckBox(QWidget *parent, int _id)
  : QCheckBox(parent), id(_id)
{
  connect(this, SIGNAL(toggled(bool)), SLOT(emitValueChanged(bool)));
}
void CtrlCheckBox::emitValueChanged  (bool val)
{
  emit valueChanged(val, id);
}
#if defined(KDE3)		    
#else  
QValidator::State DriveNameValidator::validate( QString & input, int & pos ) const
{
  if ((input.find(QChar('/')) != -1) || (input.find(QChar(' ')) != -1) || (input.find(QChar('\t')) != -1))
    {
	  return QValidator::Invalid ;
	  pos = 0;
    }
      else
	return QValidator::Acceptable;
    }
#endif
DrivesTab::DrivesTab(QDialog* parent) :QWidget(parent)
{
  QString value;
  FILE * mtab_fd ;
  struct mntent * mtab_struct;
  QString mnt_fsname,  mnt_dir;
  QString rem_or_fix, excldirs, displayname;
#if defined(KDE3)
  rx = new QRegExp( "^[^ /\t]+$" );
  validator = new QRegExpValidator( *rx, 0 );
#else
  validator = new DriveNameValidator(0);
#endif
  int nb_cols=6; 
  nb_rows=1;
  mtab_fd = setmntent (MOUNTED, "r"); 
  while ((mtab_struct = getmntent(mtab_fd)) != NULL)
    {
      mnt_fsname = QString(mtab_struct->mnt_fsname);
      if ( ! mnt_fsname.startsWith("automount") && ! mnt_fsname.startsWith("none"))
	nb_rows++;
    }
  endmntent(mtab_fd); 
  mtab_fd = setmntent (MOUNTED, "r"); 
  kapp->config()->setGroup ("Mounts");
  DriveOptionsGrid = new QGridLayout(this, nb_rows, nb_cols, GROUPBOX_SPACING, 2);
  int row=0;
  QLabel * qlab;
  DriveOptionsGrid->addWidget(qlab=new QLabel(i18n("ind. "), this), row, 0, AlignLeft | AlignVCenter);
  DriveOptionsGrid->addWidget(qlab=new QLabel(i18n("mount pt. "), this), row, 1, AlignLeft | AlignVCenter);
  DriveOptionsGrid->addWidget(qlab=new QLabel(i18n("rem. "), this), row, 2, AlignCenter);
  DriveOptionsGrid->addWidget(qlab=new QLabel(i18n("excl. "), this), row, 4, AlignCenter);
  DriveOptionsGrid->addWidget(qlab=new QLabel(i18n("name "), this), row, 5, AlignCenter);
  DriveOptionsGrid->addWidget(qlab=new QLabel(" rw ", this), row, 6, AlignCenter);
  cb_indexed=new CtrlCheckBox * [nb_rows-1];
  l_mount_point=new QLabel * [nb_rows-1];
  cb_rem_or_fix=new QCheckBox  * [nb_rows-1];
  le_excldirs=new QLineEdit  * [nb_rows-1];
  le_displayname=new  QLineEdit  * [nb_rows-1];
  cb_rw_or_ro=new QCheckBox  * [nb_rows-1];
  oldnames = new QString [nb_rows-1];
  row=1;
  while ((mtab_struct = getmntent(mtab_fd)) != NULL)
    {
      mnt_dir = QString(mtab_struct->mnt_dir);
      mnt_fsname = QString(mtab_struct->mnt_fsname);
      if ( mnt_fsname.startsWith("automount") || mnt_fsname.startsWith("none"))
	{
	}
      else
	{
	  DriveOptionsGrid->addWidget(cb_indexed[row-1]=new CtrlCheckBox(this, row-1), row, 0, AlignLeft | AlignVCenter);
  QToolTip::add(cb_indexed[row-1], i18n("Check this box if you are interested in indexing the drive on this row.") );
	  connect(cb_indexed[row-1], SIGNAL(valueChanged(bool, int)), this, SLOT(enableLine(bool,int)));
	  DriveOptionsGrid->addWidget(l_mount_point[row-1]=new QLabel(QString(mtab_struct->mnt_dir), this), row, 1, AlignLeft | AlignVCenter );
  QToolTip::add(l_mount_point[row-1], i18n("Mount point for the drive.") );
	  value = kapp->config()->readEntry(mnt_dir);
	  get_token(rem_or_fix, 1, value, "");
	  DriveOptionsGrid->addWidget(cb_rem_or_fix[row-1]=new QCheckBox(this), row, 2, AlignCenter);
  QToolTip::add(cb_rem_or_fix[row-1], i18n("Check this box if this is a drive for removable media (CD-ROM, Jaz, etc.)") );
	  cb_rem_or_fix[row-1]->setChecked(rem_or_fix.upper() == "REM");
	  get_token(excldirs, 2, value, "");
	  DriveOptionsGrid->addWidget(le_excldirs[row-1]=new QLineEdit(excldirs, this), row, 4, AlignCenter);
	  QToolTip::add(le_excldirs[row-1], i18n("Type the full paths of subdirs to exclude, separated by commas.") );
	  QString rckey_noslash=QString(l_mount_point[row-1]->text());  
	  rckey_noslash.replace(QRegExp("/"), "_"); 
	  get_token(displayname , 3, value,  rckey_noslash);
	  oldnames[row-1]= displayname; 
	  DriveOptionsGrid->addWidget(le_displayname[row-1]=new QLineEdit(displayname, this), row, 5, AlignCenter);
	  le_displayname[row-1]->setValidator( validator );
	  QToolTip::add(le_displayname[row-1], i18n("Type a name to refer to the drive.") );
	  DriveOptionsGrid->addWidget(cb_rw_or_ro[row-1]=new QCheckBox(this), row, 6, AlignCenter);
	  cb_rw_or_ro[row-1]->setChecked(hasmntopt(mtab_struct, "ro") ? FALSE : TRUE);
	  cb_rw_or_ro[row-1]->setEnabled (FALSE); 
	  cb_indexed[row-1]->setChecked(!value.isEmpty());
	  QToolTip::add(cb_rw_or_ro[row-1], i18n("Box is checked if drive is mounted read-write.") );
	  enableLine(!value.isEmpty(), row-1);
	  row++;
	}
    }
  endmntent(mtab_fd); 
  isApplied = false;
}
DrivesTab::~DrivesTab()
{
  delete [] oldnames;
#if defined(KDE3)
  delete rx;	   
#endif
  delete validator;
}
void DrivesTab::writeOptions()
{
  QString rckey, rcvalue, newname;
  for (int row = 0; row < nb_rows-1; row++)
    if (cb_indexed[row]->isChecked())
      {
	rckey = l_mount_point[row]->text();
	newname = ((le_displayname[row]->text().stripWhiteSpace().isEmpty()) ? 
		     QString(rckey).replace(QRegExp("/"), "_") 
		     : le_displayname[row]->text());
	if (!(cb_rem_or_fix[row]->isChecked()) && !(newname == oldnames[row])) 
	  {
	    QString  mess  = i18n("The display name for fixed drive %1 has been changed to %2.").arg(oldnames[row]).arg(newname);
	    KMessageBox::information( 0, mess,  i18n("Warning"));
	    if  (! tw->labellist->renDB(oldnames[row], newname, cb_rw_or_ro[row]->isChecked()))
	      {
	      le_displayname[row]->setText(oldnames[row]);
	      newname = oldnames[row];
	      }	    	  
	  }
	rcvalue  = ((cb_rem_or_fix[row]->isChecked()) ? "REM" : "FIX") ;
	rcvalue += ((le_excldirs[row]->text().stripWhiteSpace().isEmpty()) ? QString(" 0") : (" " + le_excldirs[row]->text()));
	rcvalue +=  " " + newname; 
	rcvalue +=  QString(" ") + ((cb_rw_or_ro[row]->isChecked()) ? "rw" : "ro");
	kapp->config()->setGroup ("Mounts");
	kapp->config()->writeEntry (rckey, rcvalue);
      }
    else 
      {
	rckey = l_mount_point[row]->text();
	kapp->config()->setGroup ("Mounts");
	kapp->config()->writeEntry (rckey, "");
      }
  kapp->config()->sync();
}
void DrivesTab::enableLine(bool enable, int row_nb)
{
  cb_rem_or_fix[row_nb]->setEnabled(enable);
  le_excldirs[row_nb]->setEnabled(enable);
  le_displayname[row_nb]->setEnabled(enable);
}
GeneralTab::GeneralTab(QDialog* parent) :QWidget(parent)
{
  QVBoxLayout *lo_tab = new QVBoxLayout(this, GROUPBOX_SPACING);
  lo_tab->addSpacing(DIALOG_SPACING / 2);
  lo_tab->addWidget(cb_debug=new QCheckBox(i18n("Check command lines"), this));
  cb_debug->setChecked(tw->topdebug);
  QToolTip::add(cb_debug, i18n("Check if you want to see and eventually correct the command lines sent to slocate.") );
  lo_tab->addWidget(cb_history=new QCheckBox(i18n("Keep search history"), this));
  cb_history->setChecked(tw->storehistory);
  QToolTip::add(cb_history, i18n("Check to keep a list of recent searches.") );
  lo_tab->addWidget(cb_tooltips=new QCheckBox(i18n("Show tool tips"), this));
  cb_tooltips->setChecked(tw->showtooltips);
  QToolTip::add(cb_tooltips, i18n("Uncheck to inhibit the balloon help."));
  lo_tab->addWidget(cb_seclevel=new QCheckBox(i18n("Secure locate"), this));
  cb_seclevel->setChecked(tw->seclevel);
  QToolTip::add(cb_seclevel, i18n("Check to store file permissions in databases (slower)."));
  lo_tab->addStretch();
  isApplied = false;
}
GeneralTab::~GeneralTab()
{
}
void GeneralTab::writeOptions()
{
  kapp->config()->setGroup ("Options");
  kapp->config()->writeEntry ("debug", tw->topdebug=cb_debug->isChecked());
  kapp->config()->writeEntry ("storeHistory", tw->storehistory=cb_history->isChecked());
  kapp->config()->writeEntry ("showTooltips", tw->showtooltips=cb_tooltips->isChecked());
  QToolTip::setEnabled(tw->showtooltips);
  kapp->config()->writeEntry ("seclevel", tw->seclevel=cb_seclevel->isChecked());
  kapp->config()->sync();
}
ParametersTab::ParametersTab(QDialog* parent) :QWidget(parent)
{
  QVBoxLayout *vlo_tab = new QVBoxLayout(this, GROUPBOX_SPACING);
  vlo_tab->addSpacing(DIALOG_SPACING / 2);
  QLabel * lab;
  lab = new QLabel(i18n("You must restart the application to take the new values below into account"), this);
  vlo_tab->addWidget(lab);
  vlo_tab->addSpacing(DIALOG_SPACING / 2);
  QHBoxLayout *hlo_1 = new QHBoxLayout(vlo_tab);
  lab = new QLabel(i18n("Max number of volumes:"), this);
  cnml = new QSpinBox(50, 10000, 10, this);
  cnml->setValue(MAX_LABELS);
  lab->setBuddy(cnml);
  hlo_1->addWidget(lab);
  hlo_1->addStretch();
  hlo_1->addWidget(cnml);
  QToolTip::add(cnml, i18n("Maximum number of entries this volume list can hold.") );
  QHBoxLayout *hlo_2 = new QHBoxLayout(vlo_tab);
  lab = new QLabel(i18n("Number of rows of preset buttons:"), this);
  cnnsr = new QSpinBox(0, 10, 1, this);
  cnnsr->setValue(NB_SELROWS);
  lab->setBuddy(cnnsr);
  hlo_2->addWidget(lab);
  hlo_2->addStretch();
  hlo_2->addWidget(cnnsr);
  QToolTip::add(cnnsr, i18n("Preset buttons will be displayed in as many rows.") );
  QHBoxLayout *hlo_3 = new QHBoxLayout(vlo_tab);
  lab = new QLabel(i18n("Number of columns of preset buttons:"), this);
  cnnsc = new QSpinBox(0, 10, 1, this);
  cnnsc->setValue(NB_SELCOLUMNS);
  lab->setBuddy(cnnsc);
  hlo_3->addWidget(lab);
  hlo_3->addStretch();
  hlo_3->addWidget(cnnsc);
  QToolTip::add(cnnsc, i18n("Preset buttons will be displayed in as many columns.") );
  QHBoxLayout *hlo_4 = new QHBoxLayout(vlo_tab);
  lab = new QLabel(i18n("Path of DB files:"), this);
  le_dbpath=new QLineEdit(DBPATH.join(":"), this);
  lab->setBuddy(le_dbpath);
  hlo_4->addWidget(lab);
  hlo_4->addStretch();
  hlo_4->addWidget(le_dbpath);
  QToolTip::add(le_dbpath, i18n("Colon separated list of paths where DB files are to be found."));
  vlo_tab->addStretch();
  isApplied = false;
  }
ParametersTab::~ParametersTab()
{
}
void ParametersTab::writeOptions()
{
  kapp->config()->setGroup ("Options");
  kapp->config()->writeEntry ("MAX_LABELS", MAX_LABELS=(uint)(cnml->value()));
  kapp->config()->writeEntry ("DBPATH", DBPATH=le_dbpath->text());
  kapp->config()->setGroup ("Presets");
  kapp->config()->writeEntry ("NB_SELROWS", NB_SELROWS=(uint)(cnnsr->value()));
  kapp->config()->writeEntry ("NB_SELCOLUMNS", NB_SELCOLUMNS=(uint)(cnnsc->value()));
  kapp->config()->sync();
}
OptionsDB::OptionsDB( TopWidget *parent)
  : QTabDialog( parent, 0, TRUE )
{
  dt = new DrivesTab(this);
  gt = new GeneralTab(this);
  pt = new ParametersTab(this);
  addTab(dt, i18n("Drives"));
  addTab(gt, i18n("General"));
  addTab(pt, i18n("Parameters"));
  setOkButton(i18n("OK"));
  setCancelButton(i18n("Cancel"));
  setApplyButton(i18n("Apply"));
  connect(this, SIGNAL(applyButtonPressed ()), SLOT(flagOptions())); 
  setCaption(i18n("klocate configuration"));    
}
OptionsDB::~OptionsDB( )
{
}
void OptionsDB::flagOptions()
{
  if ( dt == currentPage())  
    { 
      dt->isApplied=true;
    }
  if ( gt == currentPage())  
    { 
      gt->isApplied=true;
    }
  if ( pt == currentPage())  
    { 
      KMessageBox::information( 0, i18n("You must restart the application to take the new values below into account"),  i18n("Warning"));
      pt->isApplied=true;
    }
} 
void OptionsDB::writeOptions()
{
  if (dt->isApplied) dt->writeOptions();
  if (gt->isApplied) gt->writeOptions();
  if (pt->isApplied) pt->writeOptions();
}
DisplayCommand::DisplayCommand( QWidget *parent, QString normal_command, QString question)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  QLabel *label = new QLabel (question, this);
  whole->addWidget( label, 0, 0);
  whole->addSpacing(DIALOG_SPACING );
  le  = new QLineEdit(this);
  le->setMinimumWidth((le->fontMetrics()).width(normal_command + "coucou"));
  le->setText(normal_command);
  le->setCursorPosition (0) ;
  setFocusProxy(le);
  whole->addWidget( le, 0, 0);
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  QPushButton *ok, *cancel;
  ok = new QPushButton( i18n("Run"), this );
  ok->setDefault(true);
  connect( ok, SIGNAL(clicked()), SLOT(accept()) );
  buttons->addWidget(ok, 0, 0);
  cancel = new QPushButton( i18n("Cancel"), this );
  connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
  buttons->addWidget(cancel, 0, 0);
  whole->addLayout(buttons);
}
void SystemThread::run()
{ 
  whenIdie->result=system(command   + " 1>" + stdouttmp + " 2>" + stderrtmp );
  KApplication::postEvent( notif_handler, whenIdie );    
}
TableMessage::TableMessage( QWidget *parent, QString caption, QString header, QStringList list)
  : QDialog( parent, 0, TRUE )
{
  QVBoxLayout * whole = new QVBoxLayout( this, 0, -1, "whole");
  whole->setMargin(DIALOG_MARGIN);
  whole->addWidget(new QLabel(header, this));
 whole->addSpacing(DIALOG_SPACING / 2);
  QGridLayout * MessageGrid = new QGridLayout(whole, 8, 3);
  MessageGrid->addColSpacing(1, 2 * DIALOG_MARGIN);
  int row=0;
  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) 
    {
	    MessageGrid->addWidget(new QLabel(*it, this), row, 0, AlignLeft | AlignVCenter);
	    MessageGrid->addWidget(new QLabel(*(++it), this), row, 2, AlignLeft | AlignVCenter);
	    row++;
    }
  whole->addSpacing(DIALOG_SPACING);
  QHBoxLayout *buttons = new QHBoxLayout();
  QPushButton * ok = new QPushButton( i18n("OK"), this );
  ok->setDefault(true);
  connect( ok, SIGNAL(clicked()), SLOT(accept()) );
  buttons->addWidget(ok, 0, 0);
  whole->addLayout(buttons);
  setCaption(caption);    
}
