/*****************************************************************************
  klocate -- KDE front-end to slocate
  version 0.50.3, March 7, 2004                                                              
  Programmed by: Alexandre Oberlin                              
  Report any Bugs to: alxobr@free.fr   
  Thanks to Kevin Lindsay for programming slocate.                                                                                                                                      
     Copyright (C) version Free Software Foundation, Inc.                       
     This program is free software; you can redistribute it and/or modify    
     it under the terms of the GNU General Public License as published by    
     the Free Software Foundation; either version 2, or (at your option)     
     any later version.                                                      
     This program is distributed in the hope that it will be useful,         
     but WITHOUT ANY WARRANTY; without even the implied warranty of          
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
     GNU General Public License for more details.                            
     You should have received a copy of the GNU General Public License       
     along with this program; if not, write to the Free Software             
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               
*****************************************************************************/
/****************************************************************
** klocate 
** parse utils, classes DiskList, VolumeList, 
** VolumeEntry, DiskEntry, GuessGetString, GetUpdateList,
** DrivesTab, GeneralTab, ParametersTab, OptionsDB, GetRemLabel
** GetFixPath
****************************************************************/
#ifndef DISKLIST_H
#define DISKLIST_H
#define DEF_MAX_LABELS 120
#define DEF_NB_SELROWS 2
#define DEF_NB_SELCOLUMNS 4
#define MAX_PARTITIONS 30
#define MAX_LABEL_LENGTH 70
#define MAX_DRIVENAME_LENGTH 30
#define MAX_SECTIONNAME_LENGTH 30
#define MAX_SECTIONITEM_LENGTH 30
#define MAXLINE 200 
#define MAX_CDLABEL_LENGTH 32
#define DIALOG_MARGIN 6
#define DIALOG_SPACING 20
#define GROUPBOX_SPACING 10
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <ctype.h>
#include <dirent.h>
#include <sys/types.h>
#include <unistd.h>
#include <qpushbutton.h>
#include <qlineedit.h> 
#include <qlabel.h> 
#include <qcombobox.h>
#include <qtabdialog.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qthread.h>
#include <qevent.h> 
#include <qregexp.h>  
#include <qlayout.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlineedit.h> 
#include <qlistbox.h>
#include <qvalidator.h> 
#include <kapp.h>
#include <kconfig.h>
#include <kmessagebox.h> 
#include <kcombobox.h>
#include <mntent.h>
#include "defines.h"
extern const QString resfile;
extern QStringList DBPATH;
extern uint MAX_LABELS;
extern uint NB_SELROWS, NB_SELCOLUMNS;
extern bool MUSTQUIT;
class TopWidget;
class VolumeEntry
{
public:
  VolumeEntry(const QString _filename, const QString _directoryname);
  bool writeflag;  
  QString filename;  
  QString directoryname;
  QString displayname();
};
class DiskEntry
{
public:
  DiskEntry(const char * entry, const char * resvalue);
  QString mntpnt;
  bool remflag;
  QString excldirs;
  QString displayname;
  bool writeflag;
};
class DiskList : public QListBox
{
public:
  DiskList( QWidget *parent,  const char *name=0);
  ~DiskList();
  void showEvent ( QShowEvent * ); 
  DiskEntry * entries [MAX_PARTITIONS];
private:
  int read_entries();  
};
class VolumeList : public QListBox
{
   Q_OBJECT
public:
   VolumeList( QWidget *parent,  const char *name=0);
   ~VolumeList();
   VolumeEntry * * entries;
   QPopupMenu* popup;
   int * id; 
   void sort ( bool ascending = TRUE );
    bool renDB (QString oldname, QString newname, bool rw, QString directoryname) ;
    bool renDB (QString oldname, QString newname, bool rw) ;
 private:
    int read_entries(); 
    int readDBDIR(QString path, int old_nbe);
    void adjoinLabel(QString & new_label, QString & new_directoryname, bool _writeflag);
    QListBoxItem * contextMenuTarget;
    bool delDB(int i);
public slots:
    void selAll();
    void delSelectedDBs();
    bool deleteDB();
    bool renameDB () ;
    void statDB () ;
    void displayPopup(QListBoxItem * lbi, const QPoint & point);
    void setSel();
    void addToSel();
protected slots:
    void customEvent( QCustomEvent * e );
};
class GuessGetString : public QDialog 
{
public:
  GuessGetString( QWidget *parent, QString guess, QString question );
  ~GuessGetString( );
  QLineEdit * le;
};
class GuessGetPath : public QDialog 
{
public:
  GuessGetPath( QWidget *parent, QString guess,  QString question, QStringList combolist);
  ~GuessGetPath( );
  KComboBox * comb_dbpath;
};
class GuessGetStringandPath : public QDialog 
{
public:
  GuessGetStringandPath( QWidget *parent, QString guess1,  QString question1, QString guess2, QString question2, QStringList combolist);
  ~GuessGetStringandPath( );
  QLineEdit * le;
  KComboBox * comb_dbpath;
};
class SystemThread;
class SystemThreadFinishedEvent : public QCustomEvent
{
 public:
  SystemThreadFinishedEvent
    (int _entry_nb,
     SystemThread * _issuer,
     QString _displayname, 
     QString _remlabel,
     QString _current_db,
     bool _writeflag, 
     bool _remflag, 
     bool _db_exists
     ) 
    : QCustomEvent(QEvent::User + 1 + _entry_nb),
    entry_nb (_entry_nb),
    issuer (_issuer),
    displayname (_displayname ),
    remlabel (_remlabel ),
    current_db (_current_db ),
    writeflag (_writeflag),
    remflag (_remflag ),
    db_exists (_db_exists)
    {
    }
  ~SystemThreadFinishedEvent()
    {
    }
  int entry_nb;
  SystemThread * issuer; 
  QString displayname; 
  QString remlabel;
  QString current_db;
  bool writeflag; 
  bool remflag; 
  bool db_exists;
  int result; 
  char * stdouttmp;
  char * stderrtmp;
};
class SystemThread : public QThread
{
 public:
  SystemThread
    (QString _command, 
     QWidget * _notif_handler, 
     SystemThreadFinishedEvent * _whenIdie
     )
    : QThread(), 
    command (_command),  
    notif_handler (_notif_handler),
    whenIdie (_whenIdie)
    {
      strcpy(stderrtmp, "/tmp/klocate-errXXXXXX");
      mkstemp(stderrtmp);
      strcpy(stdouttmp, "/tmp/klocate-outXXXXXX");
      mkstemp(stdouttmp);  
      whenIdie->issuer=this;
      whenIdie->stdouttmp=stdouttmp;
      whenIdie->stderrtmp=stderrtmp;
    }
  ~SystemThread()
    {
      remove(stdouttmp);
      remove(stderrtmp);
    }
 protected:
  virtual void run();
 private:
  QString command;
  QWidget * notif_handler;
  SystemThreadFinishedEvent * whenIdie;
  char stderrtmp[25];
  char stdouttmp[25];
};
class GetUpdateList : public QDialog 
{
  Q_OBJECT
public:
  GetUpdateList (QWidget *parent);
  ~GetUpdateList( );
  int GetRemLabel(QString & remlabel, QString & dbpath, DiskEntry * current_drive);
  int GetFixPath(QString & dbpath, DiskEntry * current_drive);
  unsigned count_disklist()
    {
      return disklist->count();
    }
protected:
  DiskList * disklist;
  QPushButton *update, *done;
public slots:
  void update_db();
};
/**
 * Used to enable or disable a row of widgets.
 */
class CtrlCheckBox : public QCheckBox
{
Q_OBJECT
public:
  CtrlCheckBox  (QWidget *parent, int _id);
signals:
  void valueChanged( bool, int );
public slots:
  void emitValueChanged  (bool);
private:
 int id;
};
#if !defined(KDE3)
class DriveNameValidator : public QValidator
{
public:
  DriveNameValidator(QWidget * parent) : QValidator(parent)
    {}
  State validate( QString & input, int & pos ) const;
};		   
#endif
class DrivesTab : public QWidget
{
  Q_OBJECT
    public:
  DrivesTab(QDialog * parent);
  ~DrivesTab();
  int nb_rows;
  CtrlCheckBox ** cb_indexed;
  QLabel ** l_mount_point;
  QCheckBox ** cb_rem_or_fix;
  QLineEdit ** le_excldirs;
  QLineEdit ** le_displayname;
  QCheckBox ** cb_rw_or_ro;
  QString * oldnames;
  bool isApplied;
  void writeOptions();
 protected:
  QGridLayout * DriveOptionsGrid;
  int checked_row;
#if defined(KDE3)
  QRegExp * rx;
  QRegExpValidator * validator;		   
#else
  DriveNameValidator * validator;	   
#endif
  protected slots:
  void enableLine(bool enable, int row_nb);
};
class GeneralTab : public QWidget
{
    public:
  GeneralTab(QDialog * parent);
  ~GeneralTab();
  QCheckBox *cb_debug, *cb_history, *cb_tooltips, *cb_seclevel;
  bool isApplied;
  void writeOptions();
};
class ParametersTab : public QWidget
{
    public:
  ParametersTab(QDialog * parent);
  ~ParametersTab();
  QSpinBox *cnml, * cnnsr, *cnnsc;
  QLineEdit *le_dbpath;
  bool isApplied;
  void writeOptions();
};
class OptionsDB : public QTabDialog 
{
  Q_OBJECT
public:
  OptionsDB (TopWidget *parent);
  ~OptionsDB( );
  void writeOptions();
protected:
  GeneralTab* gt;
  DrivesTab* dt;
  ParametersTab* pt;
protected slots:
  void flagOptions();
};
class DisplayCommand : public QDialog 
{
public:
  DisplayCommand( QWidget *parent, QString guess, QString question );
  ~DisplayCommand( )
    {}
  QLineEdit * le;
};
class TableMessage : public QDialog 
{
public:
  TableMessage( QWidget *parent, QString header, QString title, QStringList list);
  ~TableMessage(){}
};
#endif 
