/*****************************************************************************
  klocate -- a KDE front-end to slocate
  version 0.50.3, March 7, 2004                                                              
  Programmed by: Alexandre Oberlin                              
  Report any Bugs to: alxobr@free.fr   
  Thanks to Kevin Lindsay for programming slocate.                                                                                                                                      
     Copyright (C) version Free Software Foundation, Inc.                       
     This program is free software; you can redistribute it and/or modify    
     it under the terms of the GNU General Public License as published by    
     the Free Software Foundation; either version 2, or (at your option)     
     any later version.                                                      
     This program is distributed in the hope that it will be useful,         
     but WITHOUT ANY WARRANTY; without even the implied warranty of          
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
     GNU General Public License for more details.                            
     You should have received a copy of the GNU General Public License       
     along with this program; if not, write to the Free Software             
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               
*****************************************************************************/
/****************************************************************
** FileList class
****************************************************************/
#include "defines.h"
#include "filelist.h"
#include "strstack.h"
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <sys/dir.h> 
#include <iostream.h>
#include <fstream.h>
#include <qapplication.h>
#include <qclipboard.h> 
#include <qtooltip.h>
#include <qtextstream.h>  
#include <qfile.h>  
#if defined(KDE3)
#include <qfont.h>
#endif
#include <klocale.h>
#include <kmessagebox.h>
extern char * stderrtmp;
extern char * stdouttmp;
extern char * stdflttmp;
enum {
  IdSearch,
  IdRun,
  IdStart,
  IdSelectFont,
  IdUndo,
  IdRedo,
#ifndef QT_NO_CLIPBOARD
  IdCut,
  IdCopy,
  IdPaste,
#endif
  IdClear,
  IdSelectAll,
  IdCount
};
#if defined(KDE3)
const QColor FileList::mycolor_error = QColor(220, 157, 157);
const QColor FileList::mycolor_output = QColor(193, 157, 202);
const QColor FileList::mycolor_abort = QColor(220, 190, 190);
const QColor FileList::mycolor_fixed_search = QColor(212, 220, 172);
const QColor FileList::mycolor_regexp_search = QColor(162, 217, 215);
#endif
FileList::FileList( QWidget *parent, const char *name )
  : KEdit( parent, name )
{
  setMinimumSize (  0,  200 ); 
#if defined(KDE3)
  setTextFormat(Qt::PlainText);
  setWordWrap(QTextEdit::NoWrap);
#endif
  setFocusPolicy(QWidget::ClickFocus);
  QFont * fl_font = new QFont();
  readFont(fl_font);
  fontChanged=false; 
  popup = new QPopupMenu( this );
  CHECK_PTR( popup );
  id = new int [ (int)IdCount ];
  id[ IdSearch ] = popup->insertItem( i18n("Find in pane"), this, SLOT(launchSearch()) );
  id[ IdRun ] = popup->insertItem( i18n("Run command"), this, SLOT(launchRun()) );
  id[ IdStart ] = popup->insertItem( i18n("Start application"), this, SLOT(launchStart()) );
  popup->insertSeparator();
  id[ IdSelectFont ] = popup->insertItem( i18n("Change font"), this, SLOT(launchSelectFont()) );
  popup->insertSeparator();
  id[ IdUndo ] = popup->insertItem( i18n( "Undo" ), this, SLOT(undo()) );
  id[ IdRedo ] = popup->insertItem( i18n( "Redo" ), this, SLOT(redo()) );
  popup->insertSeparator();
#ifndef QT_NO_CLIPBOARD
  id[ IdCut ] = popup->insertItem( i18n( "Cut" ), this, SLOT(cut()) );
  id[ IdCopy ] = popup->insertItem( i18n( "Copy" ), this, SLOT(copy()) );
  id[ IdPaste ] = popup->insertItem( i18n( "Paste" ), this, SLOT(paste()) );
#endif
  id[ IdClear ] = popup->insertItem( i18n( "Clear" ), this, SLOT(clear()) );
  popup->insertSeparator();
  id[ IdSelectAll ] = popup->insertItem( i18n( "Select All" ), this, SLOT(selectAll()) );
  installRBPopup(popup);
  QToolTip::add( this, i18n("List of results.") + "\n" + i18n("You can edit it.") + "\n" + i18n("Right-click to get a menu.") );
  connect( popup, SIGNAL(aboutToShow()), SLOT(enablePopup()) );
  setFont(*fl_font);
  delete fl_font;
}
FileList::~FileList ()
{
  if (fontChanged) writeFont();
  delete popup;
  delete [] id;
}
void FileList::readFont(QFont * fl_font)
{
  QString result, font_family; 
  int font_pointSize, font_weight; 
  bool ok; 
  kapp->config()->setGroup ("Font");
  result = kapp->config()->readEntry( QString("family"));
  if (result)
    {
      font_family = result;
      fl_font->setFamily(font_family);
    }
  result = kapp->config()->readEntry( QString("pointSize"));
  if (result)
    {
      font_pointSize = result.toUInt(&ok);
      if (ok)
	fl_font->setPointSize(font_pointSize);      
    }
  result = kapp->config()->readEntry( QString("weight"));
  if (result)
    {
      font_weight = result.toInt(&ok);
      if (ok)
	  fl_font->setWeight(font_weight);
    }
  result = (kapp->config()->readEntry("italic", "false")).upper();
  if (result  == "TRUE")
      fl_font->setItalic(true);
  else
      fl_font->setItalic(false);
}
void FileList::writeFont()
{
  kapp->config()->setGroup ("Font");
  kapp->config()->writeEntry("family", font().family());
  kapp->config()->writeEntry("pointSize", font().pointSize());
  kapp->config()->writeEntry("weight", font().weight());
  kapp->config()->writeEntry("italic", font().italic());
}
#if defined(KDE3)
void FileList::coloredAppend(const QString & text, const QColor & c )
{
  int refpara = paragraphs() -1 ; 
  moveCursor(QTextEdit::MoveEnd, false);
  append (text + "\n");
  setParagraphBackgroundColor (refpara, c);
  clearParagraphBackground(refpara + 1); 
}
#else
void FileList::insertAtEnd(const QString & text )
{
  setCursorPosition( numLines()-1, lineLength( numLines()-1 ));
  insert (text + "\n");
}
#endif

void FileList::printLocOutput(const QString output_file, const QString search_pattern, bool clearit, bool print_sparse, bool case_sens)
{
  ifstream iresult(output_file);
  if (iresult.peek() == EOF) 
    {
      iresult.close();
#if defined(KDE3)
	append("\n");
#else
	insertAtEnd("");
#endif
      return;  
    }
  if ( ! iresult )
    {
      QString mess = QString(strerror(errno));
      KMessageBox::information(parentWidget() , i18n("Could not open %1 for reading: %2").arg(output_file).arg(mess),  i18n("Error"));
      return;
    }
  if (clearit) clear();
  QStringList output_list;
  char linebuf[_POSIX_PATH_MAX + 2];  
  while (iresult.getline(linebuf, _POSIX_PATH_MAX))	   
    {
      output_list.append(linebuf);
    }
  iresult.close();
  output_list.sort();
  if (print_sparse)    
    {
      ofstream fltresult(stdflttmp);
  if ( ! fltresult )
    {
      QString mess = QString(strerror(errno));
      KMessageBox::information(parentWidget() , i18n("Could not open temp file for writing: %2").arg(mess), i18n("Error"));
      return;
    }
      QString extra_chars;
      StrStack * old_lines=new StrStack();
      char * (*str_str) (const char *HAYSTACK, const char *NEEDLE);
      if (case_sens) 
	str_str = strstr;
      else 
	str_str = strcasestr;
      for ( QStringList::Iterator it = output_list.begin(); it != output_list.end(); ++it )        
	{
	  int extrindex = old_lines->extrachars(*it);
	  extra_chars=(*it).mid(extrindex);
	  if ((extrindex != -1) && (extra_chars[0] == '/')) 
	    {
	      if (str_str(extra_chars, search_pattern)) 
		{
		  old_lines->push(*it);
		  fltresult.write( *it + "\n", strlen(*it) + 1);	   
		}
	    }
	  else 
	    {
	      old_lines->push(*it);
	       fltresult.write( *it + "\n", strlen(*it) + 1);
	    }
	}
    delete old_lines;
    fltresult.close();
}
  QString filetoread = (print_sparse) ? stdflttmp : output_file;
  {
    QFile file( filetoread ); 
    if ( file.open( IO_ReadOnly ) ) {
        QTextStream stream( &file );
#if defined(KDE3)
	append(stream.read() + "\n");
#else
	insertAtEnd(stream.read());
#endif
    }
    file.close();
  }
}
void FileList::launchSearch()
{
  search();
}
void FileList::launchSelectFont()
{
  fontChanged = true;
  selectFont();
}
void FileList::launchRun()
{
  KLineEditDlg getcmd ( i18n("Enter command:"), QString(markedText()) , this );
  getcmd.setCaption(i18n("Run command"));
  if ( !getcmd.exec() ) 
    {
      QString cmd;
      cmd = i18n("Run command canceled.");
      KMessageBox::information(this , cmd, i18n("Report"));
      return;
    }  
  QApplication::setOverrideCursor( Qt::waitCursor );
  system(getcmd.text() +  " 1>" + stdouttmp + " 2>" + stderrtmp);
#if !defined(KDE3)		   
  setCursorPosition(numLines (), 0);  
  int line, col;
  getCursorPosition (&line, &col);		   
#endif
  struct stat stats;
  stat(stderrtmp, &stats); 
  if (stats.st_size != 0) 
    {
      QString errbuf = "";
      char errline [MAXLINE + 1];
      FILE * istderr;
      if ((istderr = fopen(stderrtmp, "r") ) == NULL)
	{
	  QString mess = QString(strerror(errno));
	  KMessageBox::information(parentWidget() , i18n("Could not open %1 for reading: %2").arg(stderrtmp).arg(mess),  i18n("Error"));
	  return;
	}
      while
	(fgets(errline, MAXLINE , istderr))
	{	
	  errbuf += QString(errline);
	}
      fclose (istderr);
	  QApplication::restoreOverrideCursor();
	  KMessageBox::information( this ,  errbuf, i18n("Command error"));
	  QApplication::setOverrideCursor( Qt::waitCursor );
#if defined(KDE3)
	  coloredAppend(i18n("There was an error running `%1\'.").arg(getcmd.text()), mycolor_error);
	  append("\n");	  
#else
	  insertAtEnd(i18n("There was an error running `%1\'.").arg(getcmd.text()) + "\n");	   
#endif
    }
  stat(stdouttmp, &stats);
  if (stats.st_size != 0) 
    {
#if defined(KDE3)
      {
      coloredAppend (i18n("Output of command `%1\':").arg(getcmd.text()), mycolor_output);
      }
#else
      insertAtEnd(i18n("Output of command `%1\':").arg(getcmd.text()));
#endif
	  printLocOutput(stdouttmp, "", false, false, false);
    }
#if !defined(KDE3)		  
  setCursorPosition (line + 2 , col);
  setTopCell (line + 2); 		   
#endif
  QApplication::restoreOverrideCursor();  
}
void FileList::launchStart()
{
  KLineEditDlg getcmd ( i18n("Enter application name:"), QString(markedText()) , this );
  getcmd.setCaption(i18n("Start application"));
  if ( getcmd.exec() ) 
    {
      system(getcmd.text() + " &");
    }
  else 
    {
      QString cmd;
      cmd = i18n("Start application canceled.");
      KMessageBox::information(this , cmd, i18n("Report"));
    }   
} 
void FileList::enablePopup()
{
  popup->setItemEnabled( id[ IdSearch ], (bool)text().length() );
  popup->setItemEnabled( id[ IdRun ], TRUE );
  popup->setItemEnabled( id[ IdStart ], TRUE );
  popup->setItemEnabled( id[ IdSelectFont ], TRUE );
#ifndef QT_NO_CLIPBOARD
  popup->setItemEnabled( id[ IdCut ],
			 !isReadOnly() && hasMarkedText() );
  popup->setItemEnabled( id[ IdCopy ], hasMarkedText() );
  popup->setItemEnabled( id[ IdPaste ],
			 !isReadOnly() && (bool)QApplication::clipboard()->text().length() );
#endif
  popup->setItemEnabled( id[ IdClear ],
			 !isReadOnly() && (bool)text().length() );
  int allSelected = (bool) (text().length() == markedText().length());
  popup->setItemEnabled( id[ IdSelectAll ],
			 (bool)text().length() && !allSelected );
}
