/*****************************************************************************
  klocate -- a KDE front-end to slocate
  version 0.50.3, March 7, 2004                                                              
  Programmed by: Alexandre Oberlin                              
  Report any Bugs to: alxobr@free.fr   
  Thanks to Kevin Lindsay for programming slocate.                                                                                                                                      
     Copyright (C) version Free Software Foundation, Inc.                       
     This program is free software; you can redistribute it and/or modify    
     it under the terms of the GNU General Public License as published by    
     the Free Software Foundation; either version 2, or (at your option)     
     any later version.                                                      
     This program is distributed in the hope that it will be useful,         
     but WITHOUT ANY WARRANTY; without even the implied warranty of          
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           
     GNU General Public License for more details.                            
     You should have received a copy of the GNU General Public License       
     along with this program; if not, write to the Free Software             
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               
*****************************************************************************/
/****************************************************************
** topwidget.cpp
** klocate top widget
****************************************************************/
#include <defines.h>
#include "topwidget.h"
#include <qcursor.h> 
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <libgen.h>  
#include <errno.h>
#include "selbutton.h"
extern char * stderrtmp;
extern char * stdouttmp;
extern char * stdflttmp;
QStringList DBPATH;
uint MAX_LABELS;
uint NB_SELROWS,  NB_SELCOLUMNS;
const QString resfile = (QString)getenv("HOME") + "/.kde/share/config/klocaterc";
KLocateMain::KLocateMain  (const char *name) :
  KMainWindow (0, name)
{  
  int x, y, w, h;
  if (! kapp->geometryArgument().isNull()) 
    {
      sscanf((const char *) kapp->geometryArgument(), "%dx%d+%d+%d", &w, &h, &x, &y);
      setGeometry( x, y, w, h );
    }
  else if (!kapp->isRestored())
    {
      kapp->config()->setGroup ("Options");
      QString value = kapp->config()->readEntry ("Geometry");
      if (! value.isNull())
	{
	  sscanf((const char *) value, "%dx%d+%d+%d", &w, &h, &x, &y);
	}
      else
	{
	  QWidget *d = QApplication::desktop();                  
	  x=0, y=0, w = d->width() / 2, h = d->height() / 2;
	}
      setGeometry( x, y, w, h );
    }}
KLocateMain::~KLocateMain  ()
{ 
  if (!kapp->isRestored())
    {
      QString putres;
      putres.sprintf("%dx%d+%d+%d", width(), height(), x()-2, y()-2);
      kapp->config()->setGroup ("Options");
      kapp->config()->writeEntry ("Geometry", putres);
      kapp->config()->sync();
    }
}
TopWidget::TopWidget( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  stderrtmp = new char[strlen( "/tmp/klocate-errXXXXXX") + 1];
  strcpy(stderrtmp, "/tmp/klocate-errXXXXXX");
  mkstemp(stderrtmp);
  stdouttmp = new char[strlen( "/tmp/klocate-outXXXXXX") + 1];
  strcpy(stdouttmp, "/tmp/klocate-outXXXXXX");
  mkstemp(stdouttmp);
  stdflttmp = new char[strlen( "/tmp/klocate-fltXXXXXX") + 1];
  strcpy(stdflttmp, "/tmp/klocate-fltXXXXXX");
  mkstemp(stdflttmp);
  kapp->config()->setGroup ("Options");
  QString temp = (kapp->config()->readEntry ("storeHistory", "true")).upper();
  if (temp == "TRUE")
    storehistory = true;
  else
    storehistory = false;
  temp = (kapp->config()->readEntry ("showTooltips", "true")).upper();
  if (temp == "TRUE")
    showtooltips = true;
  else
    showtooltips = false;
  QToolTip::setEnabled(showtooltips);
  temp = (kapp->config()->readEntry ("debug", "false")).upper();
  if (temp == "TRUE")
    topdebug = true;
  else
    topdebug = false;
  temp = (kapp->config()->readEntry ("seclevel", "false")).upper();
  if (temp == "TRUE")
    seclevel = true;
  else
    seclevel = false;
  QGroupBox * gb_search = new  QGroupBox(1, Qt::Horizontal, i18n("Search"), this);
  loccmd = new LocCmd( gb_search, storehistory);
  connect( loccmd, SIGNAL(search()), SLOT(search()) );
  connect( this, SIGNAL(searchOver()), loccmd, SLOT(enableStart()) );
  QGroupBox * gb_select = new  QGroupBox(1, Qt::Horizontal, i18n("Select"), this);
  QPushButton *clear_pb = new QPushButton( i18n("Clea&rSel"), gb_select, "clear_pb" );
  QToolTip::add( clear_pb, i18n("Leave no volume selected in the volume list") );
  QPushButton *selall_pb = new QPushButton(i18n("Se&lAll"), gb_select, "selall_pb" );
  QToolTip::add( selall_pb, i18n("Select every volume in the volume list") );
  QPushButton *invsel_pb = new QPushButton(i18n("&InvSel"), gb_select, "invsel_pb" );
  QToolTip::add( invsel_pb, i18n("Invert current selections in the volume list") );
  QGroupBox * gb_manage = new  QGroupBox(1, Qt::Horizontal, i18n("Manage"), this);
  QPushButton *update_pb = new QPushButton( i18n("&Update..."), gb_manage, "update_pb" );
  QToolTip::add( update_pb, i18n("Choose volumes to update in currently mounted drives"));
  QPushButton *options_pb = new QPushButton( i18n("&Options..."), gb_manage, "options_pb" );
  QToolTip::add( options_pb, i18n("Configure your drives and set some options"));
  QPushButton *help_pb = new QPushButton( i18n("&Help..."), gb_manage, "help_pb" );
  QToolTip::add( help_pb, i18n("Find more information by reading the KLocate handbook"));
  QPushButton *quit_pb = new QPushButton( i18n("Qui&t"), gb_manage, "quit_pb");
  QToolTip::add( quit_pb, i18n("Quit KLocate"));
  connect( quit_pb, SIGNAL(clicked()), (KLocateMain*)parentWidget(), SLOT( close() ) );
  labellist = new VolumeList( this, "labellist");
  QString result;
  kapp->config()->setGroup ("Presets");
  result = kapp->config()->readEntry ("NB_SELROWS");
  if (!result)
    NB_SELROWS=DEF_NB_SELROWS;
  else
    {
      bool ok;
      NB_SELROWS=result.toUInt(&ok);
      if (!ok)
	{
	  QString msg;
	  msg =i18n("Error in %1: entry %2 should be a number.").arg(resfile).arg("NB_SELROWS");
	  KMessageBox::information( 0, msg,  i18n("Warning"));
	  NB_SELROWS=DEF_NB_SELROWS;
	}	
    }
  result = kapp->config()->readEntry ("NB_SELCOLUMNS");
  if (!result)
    NB_SELCOLUMNS=DEF_NB_SELCOLUMNS;
  else
    {
      bool ok;
      NB_SELCOLUMNS=result.toUInt(&ok);
      if (!ok)
	{
	  QString msg;
	  msg =i18n("Error in %1: entry %2 should be a number.").arg(resfile).arg("NB_SELCOLUMNS");
	  KMessageBox::information( 0, msg,  i18n("Warning"));
	  NB_SELCOLUMNS=DEF_NB_SELCOLUMNS;
	}	
    }
  if (NB_SELCOLUMNS != 0 && NB_SELROWS != 0)
    {
      selbutts = new SelButtonGroup(this, NB_SELROWS * NB_SELCOLUMNS , NB_SELCOLUMNS,  i18n("Presets"), labellist);
      QToolTip::add( selbutts, i18n("Right-click on preset button to display menu"));
      connect( selbutts, SIGNAL(setSel()), labellist, SLOT(setSel()));  
      connect( selbutts, SIGNAL(addToSel()), labellist, SLOT(addToSel()));
    }
else
  selbutts = (SelButtonGroup*) NULL;
  connect(labellist, SIGNAL(returnPressed ( QListBoxItem * )), loccmd, SIGNAL(search()));
  connect( clear_pb, SIGNAL(clicked()), labellist, SLOT(clearSelection()));
  connect( selall_pb, SIGNAL(clicked()), labellist, SLOT(selAll()) );
  connect( invsel_pb, SIGNAL(clicked()), labellist, SLOT(invertSelection()));
  connect( update_pb, SIGNAL(clicked()), this,  SLOT(update_db()));
  connect( options_pb, SIGNAL(clicked()), this,  SLOT(options()));
  connect( help_pb, SIGNAL(clicked()), this,  SLOT(slotHelp()));
  filelist = new FileList( this, "filelist" );
  QVBoxLayout *whole = new QVBoxLayout( this );
  QHBoxLayout *upper = new QHBoxLayout();
  QVBoxLayout *upper_left = new QVBoxLayout();
  QVBoxLayout *upper_right = new QVBoxLayout();
  upper_left->addWidget( gb_search, 0, 0);
  upper_left->addWidget( gb_select, 0, 0);
  upper_left->addWidget( gb_manage, 0, 0);
  upper->addLayout(upper_left);
  upper->setStretchFactor(upper_left, 0);
  if (NB_SELCOLUMNS != 0 && NB_SELROWS != 0)  upper_right->addWidget( selbutts, 0, 0);
  upper_right->addWidget( labellist);
  upper->addLayout(upper_right);
  upper->setStretchFactor(upper_right, 10);
  whole->addLayout( upper);
  whole->addWidget( filelist);
  if (labellist->count() == 0)
    {
      QString mesg = i18n("Click on \"Update...\" to index your volumes.");
      KMessageBox::information(parentWidget() ,  mesg, i18n("Tip")); 
      update_pb->setFocus();
    }
  else
    loccmd->setFocus(); 
}
TopWidget::~TopWidget()
{
  delete labellist;
  delete filelist;
  delete loccmd;
  remove(stdouttmp);
  remove(stderrtmp);
  remove(stdflttmp);
  delete(stdouttmp);
  delete(stderrtmp);
  delete(stdflttmp);
}
#include <libgen.h>  
char * readlink_malloc2 (const char *filename)  
{
  int size =  100;          
  while (1)
    {
      char *buffer = (char *) malloc (size);
      int nchars = readlink (filename, buffer, size);
      if (nchars == -1)
	{
	  return NULL;
	}
      if (nchars < size)
	{
	  buffer[nchars]='\0';
	  if ( buffer[0] == '/' )  
	    {
	      return buffer;
	    }
	  else
	    {
	      char * tempfilename = (char *) malloc (strlen(filename)+1);
	      strcpy(tempfilename, filename);
	      char * direcname = dirname(tempfilename);
	      char * target = (char *) malloc(strlen(direcname) + 2 + strlen(buffer));
	      strcpy(target, direcname);
	      free(tempfilename);
	      strcat(target, "/");
	      strcat(target, buffer);
	      free (buffer);
	      return target;
	    }
	}
      free (buffer);
      size *= 2;
    }
}
int utarget2 (char ** utarget, int * pusize, const char * filename)
{
  struct stat *buf  = (struct stat *) malloc (sizeof (struct stat));
  *utarget= (char *) malloc (strlen (filename) + 1);
  strcpy(*utarget, filename);
  char * ult;
  int result;
  while (1)
    {
      if ((result = lstat(*utarget, buf)) == -1)
	{
	  QString mess = QString(strerror(errno));
	  KMessageBox::information( 0, i18n("Could not lstat %1: %2").arg(*utarget).arg(mess),  i18n("Warning"));
	  free(buf);
	  free(*utarget);
	  *utarget = NULL;
	  *pusize = -1;
	  return result;
	}
      else
	{   
	  if (S_ISLNK (buf->st_mode))
	    {		  
	      ult = readlink_malloc2(*utarget);
	      free(*utarget);
	      *utarget = (char *) malloc (strlen (ult) + 1);
	      strcpy(*utarget, ult);
	      free(ult);
	    }
	  else
	    {
	      *pusize = buf->st_size;
	      free (buf);
	      return result;
	    }
	}}
}
void TopWidget::search()
{
  bool nothing_sel = true;
  for (uint i = 0 ; i< labellist->count(); i++)
    {
      if (labellist->isSelected(i))
	{
	nothing_sel = false;
	break;
	}
    }
  if (nothing_sel) 
    {
      KMessageBox::information(this , i18n("Please select volumes to search in the volume list."), i18n("Tip"));
      emit searchOver();      
      return;
    }
  loccmd->remember();
  int entrylen=loccmd->value().length();
  if ((entrylen > MAX_SEARCH_STRING) || (entrylen == 0))
    {
      KMessageBox::information( this, i18n("Search string too long or too short!"), i18n("Tip"));
      emit searchOver();
      return;
    }
  filelist->clear();
  VolumeEntry * current_volume;
  QString current_file, current_db, cmd_buffer, cmd_buffer_noredir;
  struct stat  stats;  
  char * target2 = NULL; 
  int usize = -1;
  int ustatOK = -1;
  loccmd->stopit=FALSE;
  for (uint i = 0 ; i< labellist->count(); i++)
    {
      if (labellist->isSelected(i))
	{ 
	  current_volume = labellist->entries[i];
	  current_file=current_volume->directoryname + current_volume->filename;
	    target2 = NULL;
	  usize = -1;
	  ustatOK = utarget2 (&target2, &usize, current_file.latin1());
	  current_db = QString(target2);  
	  free(target2);
	  if  (ustatOK != EXIT_SUCCESS )
	    {
	      QString mess = QString(strerror(errno));
	      KMessageBox::information( 0, i18n("Can't access %1 for reading: %2").arg(current_file).arg(mess),  i18n("Error"));
	      continue;
	    }
	  else if (usize == 0)
	    {
	      KMessageBox::information( 0, i18n("Can't access %1 for reading: zero length file probably locked by another process.").arg(current_file),  i18n("Error"));
	      continue;
	    }
	  if (loccmd->cb3->isOn()) 
	    cmd_buffer_noredir = QString(loccmd->cb->isOn() ? QString("slocate ") : QString("slocate -i ")) 
	      + " -d '" + current_db  + "' -r \"" +  loccmd->value() + "\"";
	  else 
	    cmd_buffer_noredir = QString(loccmd->cb->isOn() ? QString("slocate ") : QString("slocate -i ")) 
	      + " -d '" + current_db  + "' "  + loccmd->value();
	  cmd_buffer = cmd_buffer_noredir + " 1>" + stdouttmp + " 2>" + stderrtmp;
	  if (topdebug)
	    {
	      DisplayCommand * dpcom;
	      dpcom = new DisplayCommand(this, cmd_buffer_noredir, i18n("Edit search command if you wish:"));
	      if ( dpcom->exec() ) 
		{
		  cmd_buffer_noredir = dpcom->le->text().stripWhiteSpace();
		  cmd_buffer = cmd_buffer_noredir + " 1>" + stdouttmp + " 2>" + stderrtmp;
		  delete dpcom;
		}
	      else 
		{
		  QString cmd;
		  cmd = i18n("Search for string `%1' in volume %2 canceled.").arg(loccmd->value()).arg(current_volume->displayname());
		  if (loccmd->cb3->isOn()) cmd = "Regexp " + cmd;
		  KMessageBox::information(this , cmd, i18n("Report"));
		  cmd_buffer="";
		  cmd_buffer_noredir="";
		  delete dpcom;
		  continue ;
		}
	    }
	  QApplication::setOverrideCursor( Qt::waitCursor );
#if defined(KDE3)
	  if (loccmd->cb3->isOn()) 
	    { 
	      filelist->coloredAppend (i18n("### Searching regexp `%1' on volume %2 ###")
				       .arg(loccmd->value()).arg(current_volume->displayname()), FileList::mycolor_regexp_search);
	    }	     
	  else
	    {
	      filelist->coloredAppend (i18n("### Searching `%1' on volume %2 ###")
				       .arg(loccmd->value()).arg(current_volume->displayname()), FileList::mycolor_fixed_search);
	    }
#else
	  if (loccmd->cb3->isOn()) 
	    filelist->insertAtEnd (i18n("### Searching regexp `%1' on volume %2 ###")
				  .arg(loccmd->value()).arg(current_volume->displayname()));
	  else
	    filelist->insertAtEnd (i18n("### Searching `%1' on volume %2 ###")
				  .arg(loccmd->value()).arg(current_volume->displayname()));
#endif	  
	  int result = 0;
	  system(cmd_buffer);  
	  stat(stderrtmp, &stats);
	  if (stats.st_size != 0) 
	    {
	      QString errbuf = "";
	      char errline [MAXLINE + 1];
	      FILE * istderr;
		if ((istderr = fopen(stderrtmp, "r") ) == NULL)
		  {
		    QString mess = QString(strerror(errno));
		    KMessageBox::information(parentWidget() , i18n("Could not open %1 for reading: %2")
					     .arg(stderrtmp).arg(mess),  i18n("Error"));
		    emit searchOver();		    
		    return;
		  }
	      while
		(fgets(errline, MAXLINE , istderr))
		{		
		  if  (strstr(errline, i18n("is more than 8 days old")))
		    {
		    if (current_volume->writeflag == false)
		      ;  
		    else
		      errbuf += QString(errline);
		    }
		  else 
		    {
		      errbuf += QString(errline);
		      result=EXIT_FAILURE;
		    }
		}
	      fclose (istderr);
	      if (errbuf != "") KMessageBox::information( this ,  errbuf, i18n("Error") );
	    }
	  if (result == EXIT_FAILURE) 
#if defined(KDE3)
	    {
	      filelist->coloredAppend(i18n("There was an error."), FileList::mycolor_error);
	      filelist->append("\n");
	    }
#else  
	    filelist->insertAtEnd(i18n("There was an error.") + "\n");
#endif
	  else
	    {
	  if (loccmd->cb3->isOn()) 
	    filelist->printLocOutput(stdouttmp, loccmd->value(), false, false, loccmd->cb->isOn());
	  else
	    filelist->printLocOutput(stdouttmp, loccmd->value(), false, loccmd->cb2->isOn(), loccmd->cb->isOn());
	    }
	  QApplication::restoreOverrideCursor();
	}
      kapp->processEvents(); 
      if (loccmd->stopit) 
	{
#if defined(KDE3)
		filelist->coloredAppend(i18n("!!! SEARCH ABORTED AT USER'S REQUEST !!!"), FileList::mycolor_abort);
		filelist->append("\n");
#else
		filelist->insertAtEnd(i18n("!!! SEARCH ABORTED AT USER'S REQUEST !!!") + "\n");
#endif
	  KMessageBox::information( this , i18n("Search aborted at user's request.") , i18n("Report") );
	  loccmd->stopit=false; 
	  break; 
	}
    }
  emit searchOver();
}
void TopWidget::update_db()
{
  gul = new GetUpdateList(this);
  if (gul->count_disklist() == 0)
    {
      QString mesg = i18n("You must configure your klocaterc file before you can create databases.");
      KMessageBox::information(this ,  mesg, i18n("Tip")); 
      delete gul;
      options();
    }
  else
    {
	gul->exec();
      delete gul;
    }
}
void TopWidget::options()
{
  odb = new OptionsDB(this);
  int result = odb->exec();
  if (result == QDialog::Accepted) 
    {
      odb->writeOptions();
    }
    delete odb;
}
void TopWidget::slotHelp()
{
  KPopupMenu *help = ((KMainWindow*)(kapp->mainWidget()))-> helpMenu("", false);
  help->exec(QCursor::pos());
}
void TopWidget::FileMB(QString dfile)
{
  struct stat  stats;
  stat(dfile, &stats);
  if (stats.st_size != 0) 
    {
      char * outbuf = new char [stats.st_size + 1];
      FILE * idfile;
      if ((idfile = fopen(dfile, "r") ) == NULL)
	{
	  QString mess = QString(strerror(errno));
	  KMessageBox::information(parentWidget() , i18n("Could not open %1 for reading: %2").arg(dfile).arg(mess),  i18n("Error"));
	return;
	}
      int pnt = 0;
      while
	(fgets(outbuf+pnt, 99 , idfile))
	pnt+= strlen(outbuf+pnt);
      fclose (idfile);
      KMessageBox::information( this , outbuf, i18n("Message"));
      delete [] outbuf;
    }
}
void TopWidget::customEvent( QCustomEvent * e )
{
  if ( e->type() >= QEvent::User + 1 ) 
    {
      SystemThreadFinishedEvent * ce = (SystemThreadFinishedEvent*)e;
      SystemThreadFinishedEvent * ce2 = new SystemThreadFinishedEvent
	(
	 ce->entry_nb, 
	 ce->issuer,
	 ce->displayname,
	 ce->remlabel,
	 ce->current_db,
	 ce->writeflag,
	 ce->remflag,
	 ce->db_exists
	 );
      ce2->result=ce->result;
      KApplication::postEvent( labellist, ce2 );    
      FileMB(ce->stderrtmp);
      FileMB(ce->stdouttmp);
      QString final_comment;
      if (ce->result == 0)
	{
	  if (ce->db_exists)
	    final_comment = i18n("Updated volume %1 to DB %2.").arg(ce->displayname).arg(ce->current_db);
	  else
	    final_comment = i18n("Created index file %1 for volume %2.").arg(ce->current_db).arg(ce->displayname);
	  if (final_comment != "") 
	    KMessageBox::information( this, final_comment, i18n("Report"));
	  else
	    KMessageBox::information( this, i18n("No database was updated."), i18n("Report"));
	}
      delete ce->issuer;
    }}
